@@ -203,43 +203,59 @@ class DjangoLexer(RegexLexer):
     aliases = ['django', 'jinja']
     mimetypes = ['application/x-django-templating', 'application/x-jinja']
 
+    flags = re.M | re.S
+
     tokens = {
         'root': [
             (r'[^\{]+', Other),
             (r'\{\{', Comment.Preproc, 'var'),
-            # jinja comments
-            (r'\{\*.*?\*\}', Comment),
+            # jinja/django comments
+            (r'\{[*#].*?[*#]\}', Comment),
             # django comments
-            (r'(\{\%)(\s*)(comment)(\s*)(\%\})(.*?)'
-             r'(\{\%)(\s*)(endcomment)(\s*)(\%\})',
+            (r'(\{\%)(\-?\s*)(comment)(\s*\-?)(\%\})(.*?)'
+             r'(\{\%)(\-?\s*)(endcomment)(\s*\-?)(\%\})',
              bygroups(Comment.Preproc, Text, Keyword, Text, Comment.Preproc,
                       Comment, Comment.Preproc, Text, Keyword, Text,
                       Comment.Preproc)),
-            (r'(\{\%)(\s*)([a-zA-Z_][a-zA-Z0-9_]*)',
+            # raw jinja blocks
+            (r'(\{\%)(\-?\s*)(raw)(\s*\-?)(\%\})(.*?)'
+             r'(\{\%)(\-?\s*)(endraw)(\s*\-?)(\%\})',
+             bygroups(Comment.Preproc, Text, Keyword, Text, Comment.Preproc,
+                      Text, Comment.Preproc, Text, Keyword, Text,
+                      Comment.Preproc)),
+            # filter blocks
+            (r'(\{\%)(\-?\s*)(filter)(\s+)([a-zA-Z_][a-zA-Z0-9_]*)',
+             bygroups(Comment.Preproc, Text, Keyword, Text, Name.Function),
+             'block'),
+            (r'(\{\%)(\-?\s*)([a-zA-Z_][a-zA-Z0-9_]*)',
              bygroups(Comment.Preproc, Text, Keyword), 'block'),
             (r'\{', Other)
         ],
         'varnames': [
-            (r'[a-zA-Z][a-zA-Z0-9_]*(\.[a-zA-Z0-9_]+)*',
-             Name.Variable),
             (r'(\|)(\s*)([a-zA-Z_][a-zA-Z0-9_]*)',
              bygroups(Operator, Text, Name.Function)),
+            (r'(is)(\s+)(not)?(\s+)?([a-zA-Z_][a-zA-Z0-9_]*)',
+             bygroups(Keyword, Text, Keyword, Text, Name.Function)),
+            (r'(_|(?:true|false|undefined|null))\b', Keyword.Pseudo),
+            (r'(in|as|reversed|recursive|not|and|or|with|is|accepting)\b', Keyword),
+            (r'(loop|block|forloop)\b', Name.Builtin),
+            (r'[a-zA-Z][a-zA-Z0-9_]*', Name.Variable),
+            (r'\.[a-zA-Z0-9_]+', Name.Variable),
             (r':?"(\\\\|\\"|[^"])*"', String.Double),
             (r":?'(\\\\|\\'|[^'])*'", String.Single),
+            (r'([{}()\[\]+\-*/,:]|[><=]=?)', Operator),
             (r"[0-9](\.[0-9]*)?(eE[+-][0-9])?[flFLdD]?|"
              r"0[xX][0-9a-fA-F]+[Ll]?", Number),
         ],
         'var': [
             (r'\s+', Text),
-            include('varnames'),
-            (r'\}\}', Comment.Preproc, '#pop')
+            (r'(\-?)(\}\})', bygroups(Text, Comment.Preproc), '#pop'),
+            include('varnames')
         ],
         'block': [
             (r'\s+', Text),
-            (r'(in|as|reversed|not|count|and|or|with|equals|accepting)\b',
-             Keyword),
+            (r'(\-?)(\%\})', bygroups(Text, Comment.Preproc), '#pop'),
             include('varnames'),
-            (r'\%\}', Comment.Preproc, '#pop'),
             (r'.', Punctuation)
         ]
     }
@@ -403,7 +419,7 @@ class MakoLexer(RegexLexer):
             (r'<%(?=(include|inherit|namespace|page))', Comment.Preproc, 'ondeftags'),
             (r'(<%(?:!?))(.*?)(%>)(?s)',
              bygroups(Comment.Preproc, using(PythonLexer), Comment.Preproc)),
-            (r'(\$\{)(.*?)(\})',
+            (r'(\$\{!?)(.*?)(\})(?s)',
              bygroups(Comment.Preproc, using(PythonLexer), Comment.Preproc)),
             (r'''(?sx)
                 (.+?)               # anything, followed by:
